var path = require('path')
var fs = require('fs')
var mustache = require('mustache')
var mjml = require('mjml')
var marked = require('./marked')
var url = require('url')
var fetch = require('node-fetch')

const readFile = require('util').promisify(fs.readFile)
const writeFile = require('util').promisify(fs.writeFile)

function getDir() {
  var copy = (' ' + module.parent.filename).slice(1).split('/')
  copy.pop()
  return copy.join('/')
}

function load(file, isMarkdown = false) {
  if(url.parse(file).protocol) {
    return fetch(file).then(resp => resp.text())
  }

  let localPath = isMarkdown ? path.join(getDir(), 'partials', file + '.md') : file

  return readFile(localPath)
    .then(file => file.toString('utf-8'))
}

function wrap(inner, markup) {
  var prefix = markup.map(mark => `<${mark}>`).join('\n')
  var suffix = markup.map(mark => `</${mark}>`).reverse().join('\n')
  return prefix + inner + suffix
}

function unwrap(mjml, markup = ['mjml']) {
  return markup.reduce((acc, mark) => {
    return acc.replace(`<${mark}>`, '').replace(`</${mark}>`, '')
  }, mjml)
}

function combine() {
  return [].slice.call(arguments).join('\n')
}

function createSchema(list) {
  var partials = list.filter(item => item[0] === '>') 
  var files = partials.map(partial => load(partial[1], true))
  return Promise.all(files)
    .then(arr => arr.reduce((acc, val, i) => {
      var partial = partials[i]
      acc[partial[1]] = val;
      return acc
    }, {}))
}

function Template(params) {

  Object.assign(this, params)
}

Template.prototype.loadMjml = function() {
  return load(this.mjml).then(template => {
    this.template = template;
    return template;
  })
}

Template.prototype.loadMarkdown = function() {
  return load(this.markdown, true)
}

Template.prototype.parse = function(partials) {
  for(var key in partials) {
    partials[key] = wrap(marked(partials[key]), ['mj-section', 'mj-column'])
  }
  var mjml = mustache.render(this.template, this.tags, partials)
  return this.injectStyles(mjml).promise
}

Template.prototype.loadPartials = function() {
  this.promise = this.loadMjml()
    .then(mjml => mustache.parse(mjml))
    .then(list => createSchema(list))
      
  return this
}

Template.prototype.injectStyles = function(mjml) {
  this.promise = load(this.stylesheet)
    .then(raw => wrap(raw, ['mj-head', 'mj-style']))
    .then(styles => combine(styles, unwrap(mjml)))
    .then(content => wrap(content, ['mjml']))
  return this
}

Template.prototype.save = function (mjml) {
  return writeFile(path.join(getDir(), 'test.html'), mjml)
}

Template.prototype.send = function() {
   this.promise.then(html => this.mailgun.messages().send({}))
}

module.exports = Template